<?php
namespace Fiscal\Logic\NotaFiscalEletronica\TransmissaoNfEletronica;

use Financeiro\Entities\Contareceber;
use Fiscal\Entities\Dadosnf;
use Fiscal\Entities\Itemproimpos;
use Fiscal\Entities\Notafiscal;
use Fiscal\Logic\SimplesNacional\ValidacaoCFOP;
use Geral\Entities\Filial;
use Geral\Entities\Itemproduto;
use Geral\Entities\Paramempresa;
use Geral\Entities\Paramgeral;
use Geral\Entities\Produto;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\PictureProduto;
use Geral\Util\Util;
use Indeterminado\Entities\Adicdecimpnfe;
use Indeterminado\Entities\Impostonfe;
use Indeterminado\Entities\Nfprodrural;
use Indeterminado\Entities\Relacnflotenfe;
use Indeterminado\Logic\CfopPadrao\ValidaCFOPpadrao;


class ValidacaoNfEletronica
{
    const ID_STATUS_NF_TRANSMITIDA = 'T';
    const ID_STATUS_NF_APROVADA = 'A';
    const ID_STATUS_NF_EMITIDA = 'E';
    const ID_STATUS_NF_CANCELADA = 'C';
    const ID_SAIDA = 'S';
    const ID_ENTRADA = 'E';
    const ARRAY_DE_ERROS = 'ERROS_TRANSMISSAO_NFE';
    const TIPO_OPERAC_COMPLEMENTO = '03';
    const ID_CONTINGENCIA_SCAN = '3';
    const ID_IMPOSTO_ISSQN = '31';
    const ID_MODELO_DOC_NFE = '55';
    const ID_PAIS_BRASIL_BCO_CENTRAL = '1058';
    const QTD_CHAR_CPF = 11;
    const QTD_CHAR_CNPJ = 14;
    const QTD_CHAR_MUNI_IBGE = 7;
    const VERSAO = 'V_300';
    const REGIME_NORMAL = '3';

    private $notasfiscais = array();
    private $modulo;
    private $arrayErros = array();
    private $arrayMsgErros = array(
        'LANC_BLOQ'                  => 'Operação bloqueada. Transmissão sendo realizada por outro usuário.',
        'COD_EST_EMI'                => 'Código do IBGE do estado do emitente deve ser cadastrado.',
        'NAT_OPE_DEV'                => 'Natureza da operação deve ser cadastrado.',
        'SERIE_DEV'                  => 'Código da série de nota fiscal eletrônica deve ser cadastrado.',
        'COD_MUN_EMI'                => 'Código do IBGE do município do emitente deve ser cadastrado.',
        'SERIE_ORIGEM'               => 'Série da nota fiscal eletrônica para o documento de origem deve ser cadastrada.',
        'ISSQN_IM_DEV'               => 'Para operações com tributação de ISSQN a inscrição municipal é obrigatória.',
        'CNAE_DEV'                   => 'Código CNAE (Classificação Nacional de Atividades Econômicas) da empresa deve ser cadastrado.',
        'COD_PAIS_EMI'               => 'Código do Banco Central do país do emitente deve ser cadastrado.',
        'END_EMI_DEV'                => 'Endereço do Emitente deve ser cadastrado.',
        'BAI_EMI'                    => 'Bairro do emitente deve ser cadastrado.',
        'MUN_EMI'                    => 'Município do emitende deve ser cadastrado.',
        'COD_PAIS_DES'               => 'Código do Banco Central do país do destinatário deve ser cadastrado.',
        'END_DES_DEV'                => 'Endereço do destinatário deve ser cadastrado.',
        'BAI_DES'                    => 'Bairro do destinatário deve ser cadastrado.',
        'COD_MUN_DES'                => 'Código IBGE do Município do destinatário deve ser cadastrado.',
        'MUN_DES'                    => 'Município do destinatário deve ser cadastrado.',
        'IE_TRA_INV'                 => 'IE da transportadora inválido. Inscrição deve conter no mínimo 2 e no máximo 14 caracteres.',
        'UF_VEI_TRA'                 => 'UF do veículo da transportadora %s deve ser cadastrado.',
        'PLA_VEI_TRA'                => 'Placa do veículo da transportadora deve ser cadastrada.',
        'COD_NCM_DEV'                => 'Código NCM (Nomenclatura Comum do Mercosul) do produto %s deve ser informado.',
        'COD_NCM_INC'                => 'Código NCM (Nomenclatura Comum do Mercosul) do produto %s inválido. Código deve conter 8 caracteres para um código completo ou 2 caracteres quando informado somente o capítulo.',
        'COD_CFOP_DEV'               => 'CFOP para o produto %s deve ser cadastrado.',
        'COD_CST_INV'                => 'CST do produto %s inválido. Verificar o primeiro dígito do CST no cadastro do cliente, ou do fornecedor ou da inscrição estadual.',
        'DAT_FAB_LOT'                => 'Data de fabricação do lote relacionado ao produto %s deve ser cadastrado.',
        'COD_CEAN_INV'               => 'Código EAN do produto %s inválido. Código EAN deve conter 8, 12, 13 ou 14 posições.',
        'COD_CSOSN_DEV'              => 'CSOSN (Código de Situação da Operação no Simples Nacional) do produto %s deve ser cadastrado.',
        'CST_ICMS_DEV'               => 'CST do ICMS para o produto %s deve ser cadastrado.',
        'MOD_BC_DEV'                 => 'Modalidade da Base de Cálculo do ICMS do produto %s deve ser cadastrado.',
        'MOD_BCST_DEV'               => 'Modalidade da Base de Cálculo do ICMS do Substituto Tributário do produto %s deve ser cadastrado.',
        'CLA_SER_DEV'                => 'Classificação do serviço do produto %s deve ser cadastrada.',
        'COD_TRIB_ISSQN_DEV'         => 'Código de Tributação do ISSQN para o produto %s deve ser informado.',
        'ICMS_ST_N_RELAC'            => 'Atenção! O imposto ICMS ST do produto %s não contém relacionamento fiscal.',
        'ICMS_OU_ISS_DEV'            => 'ICMS ou ISSQN para o Produto %s deve ser cadastrado.',
        'CST_IPI_DEV'                => 'CST do IPI para o produto %s deve ser cadastrado.',
        'CONFLITO_INCIDENCIA'        => 'Conflito na incidência de impostos para o produto %s. Produtos com ISSQN não poderão ter a incidência de ICMS nem de IPI.',
        'CST_PIS_DEV'                => 'CST do PIS para o produto %s deve ser cadastrado.',
        'CST_COF_DEV'                => 'CST do COFINS para o produto %s deve ser cadastrado.',
        'SEM_FILI_CENT'              => 'Filial não possui centralizadora relacionada.',
        'DT_APROV_DEV'               => 'Data de processamento da filial centralizadora deve ser informada.',
        'CST_PIS_INCORRETO'          => 'Item %s com Código da Situação Tributária para o imposto PIS inválido.',
        'PIS_DEV'                    => 'Produto %s sem incidência de PIS.',
        'COF_DEV'                    => 'Produto %s sem incidência de COFINS.',
        'MOD_FRETE_DEV'              => 'Modalidade do frete deve ser cadastrado.',
        'MOD_FRETE_INV'              => 'Modalidade do frete inválido.',
        'TIPO_EMIS_INV'              => 'Tipo de emissão da NF-e inválido.',
        'ICMS_N_RELAC'               => 'Atenção! O imposto ICMS do produto %s não contém relacionamento fiscal.',
        'ISSQN_N_RELAC'              => 'Atenção! O imposto ISSQN do produto %s não contém relacionamento fiscal.',
        'PIS_N_RELAC'                => 'Atenção! O imposto PIS do produto %s não contém relacionamento fiscal.',
        'COFINS_N_RELAC'             => 'Atenção! O imposto COFINS do produto %s não contém relacionamento fiscal.',
        'CST_COF_INCORRETO'          => 'Item %s com Código da Situação Tributária para o imposto COFINS inválido.',
        'CFOP_IMP_INC_IPI'           => 'CFOP do produto %s de Importação e não informado dados do imposto IPI.',
        'CFOP_IMP_INC_II'            => 'CFOP do produto %s de Importação e não informado dados do imposto II.',
        'INSC_SUFRAMA_DEV'           => 'Inscrição SUFRAMA deve ser informada na venda com isenção para Zona Franca de Manaus.',
        'CFOP_INVA_SUFRAMA'          => 'CFOP do produto %s inválido: CFOP de operação isenta para ZFM deve ser 6109 ou 6110.',
        'CEP_EMIT_DEV'               => 'CEP do emitente deve ser informado.',
        'CEP_EMIT_INC'               => 'CEP do emitente inválido: CEP deve conter 8 caracteres e todos eles devem ser numéricos.',
        'CEP_DEST_DEV'               => 'CEP do destinatário deve ser informado.',
        'CEP_DEST_INC'               => 'CEP do destinatário inválido: CEP deve conter 8 caracteres e todos eles devem ser numéricos.',
        'COD_CFOP_INC'               => 'CFOP do produto %s inválido. Favor informar um CFOP válido.',
        'PLA_VEI_TRA_INC'            => 'Placa do veículo da transportadora %s inválido. Formato da placa deve ser XXX9999.',
        'DUPLI_MAIOR_120'            => 'Erro na quantidade de duplicatas: O máximo permitido pela SEFAZ são de 120 duplicatas.',
        'ADICOES_MAIOR_120'          => 'Erro na quantidade de adições: O máximo permitido pela SEFAZ são de 100 adições por produto.',
        'SERIESCAN_DEV'              => 'Código da série de nota fiscal eletrônica (SCAN) deve ser cadastrado.',
        'SERIESCAN_ORIGEM'           => 'Série da nota fiscal eletrônica (SCAN) para o documento de origem deve ser cadastrada.',
        'CST_IPI_INCORRETO'          => 'Item %s com Código da Situação Tributária para o imposto IPI inválido.',
        'ICMSST_N_CAD'               => 'ICMS ST para o Produto %s deve ser cadastrado.',
        'COD_IBGE_PROD_RURAL'        => 'Código IBGE do estado do produtor rural da nota %s (CPF/CNPJ: %s, Série: %s) deve ser informado.',
        'CDMODDOCFISC_INCORRETO'     => 'Código do modelo do documento fiscal inválido. Utilizar o código 55 para identificação da NF-e.',
        'NF_EMIT_APROV'              => 'Operação Bloqueada! Nota fiscal %s sem passar pelo processo eletrônico.',
        'COD_CSOSN_INV'              => 'CSOSN (Código de Situação da Operação no Simples Nacional) do produto %s inválido.',
        'NFRELACLOTE'                => 'Operação Bloqueada! Nota fiscal relacionada ao lote Empresa: %s Número: %s.',
        'CDMUNEMI_INV'               => 'Código do IBGE do município do emitente inválido.',
        'CNPJ_CPF_PRODRURAL_INV'     => 'CNPJ/CPF da NF referenciada de produtor inválido.',
        'CNPJ_CPF_PRODRURAL_DEVE'    => 'CNPJ/CPF da NF referenciada de produtor deve ser informado.',
        'CNPJ_CPF_EMITENTE_INV'      => 'CNPJ/CPF do remetente inválido.',
        'CNPJ_CPF_EMITENTE_DEVE'     => 'CNPJ/CPF do remetente deve ser informado.',
        'CNPJ_CPF_DESTINATARIO_INV'  => 'CNPJ/CPF do destinatário inválido.',
        'CNPJ_CPF_DESTINATARIO_DEVE' => 'CNPJ/CPF do destinatário deve ser informado.',
        'CNPJ_CPF_TRANSP_INV'        => 'CNPJ/CPF do transportador inválido.',
        'QVOL_DEVE'                  => 'Quantidade de volumes transportados deve ser informado.',
        'HORVERAO_DEVE'              => 'Operação Bloqueada! Período do horário de verão deve ser informado.',
        'DIFUTCFUSO_DEVE'            => 'Operação Bloqueada! Diferença do fuso horário deve ser informado.',
        'DATA_MAIOR'                 => 'Operação Bloqueada! Data de emissão deve ser menor ou igual a data de saída.',
        'DTEMISSAO_DIFERE_BD'        => 'Operação Bloqueada! Data de emissão diferente da data corrente %s.',
        'VRVENC_DIFERE_VR_NOTA'      => 'Operação Bloqueada! Valor total dos vencimentos difere do valor total da Nota Fiscal.',
    );

    function __construct($notasfiscais, $modulo)
    {
        try {
            $this->notasfiscais = $notasfiscais;
            $this->modulo       = $modulo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaNFe()
    {
        /** @var $notafiscal Notafiscal */
        foreach ($this->notasfiscais as $notafiscal) {
            if ($this->modulo == self::VERSAO) {
                $corpoNF = self::getCorpoNf($notafiscal);

                /** @var  $paramgeral Paramgeral */
                $paramgeral = Paramgeral::findOneByNrorg(Ambiente::getNrorg());

                try {
                    Notafiscal::getRepository()->consultaBloqueiaNotaFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
                } catch (\Exception $e) {
                    $arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                          'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                          'MENSAGEM'   => $this->arrayMsgErros['LANC_BLOQ']);
                }

                try{
                    $iderroaviso       = Notafiscal::getRepository()->getIdErroAviso($notafiscal->getCdfilial());
                    $valorTotalImposto = Notafiscal::getRepository()->getValorTotalImposto(
                        $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $iderroaviso[0]["IDERROAVISOFIL"], $paramgeral->getNrdecitotanf() ? : 2
                    );
                    $valorSubtracaoVencimentos = Notafiscal::getRepository()->getValorSubtracaoVencimentos(
                        $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getIdentrsaidop(), $notafiscal->getCdtipooper()
                    );

                    $valorTotalVencimentos = Notafiscal::getRepository()->getValorTotalVencimentos(
                        $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $valorTotalImposto[0]["V_VRIMPOSTOT"], $valorSubtracaoVencimentos[0]["VRSUBTRAIVENC"]
                    );
                    if($valorTotalVencimentos[0]["VRDIFERENCA"] != 0)
                        throw new \Exception();
                }catch (\Exception $e){
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                        'MENSAGEM'   => $this->arrayMsgErros['VRVENC_DIFERE_VR_NOTA']);
                }

                $relacnflotenfe = Relacnflotenfe::findOneByCdfilialAndNrlanctonfAndIdstatusrelnfeAndNrorg(
                    $notafiscal->getCdfilial(),
                    $notafiscal->getNrlanctonf(), 'T', $notafiscal->getNrorg()
                );
                if ($relacnflotenfe instanceof Relacnflotenfe) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['NFRELACLOTE'], array($notafiscal->getCdemplotenfe(), $notafiscal->getNrlotenfe())));
                }

                $consisteFiscal = Notafiscal::getRepository()->consisteFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
                if (is_array($consisteFiscal) && !empty($consisteFiscal)) {
                    foreach ($consisteFiscal as $erro) {
                        if (empty($erro['CDIMPOSTO'])) {
                            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                        'MENSAGEM'   => $erro['DSCONSISTENCIA'] . ' (Produto: ' . $erro['CDARVPROD']);
                        } else {
                            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                        'MENSAGEM'   => $erro['DSCONSISTENCIA'] . ' (Produto: ' . $erro['CDARVPROD'] . ' - Imposto: ' . $erro['CDIMPOSTO']);
                        }
                    }
                }

                switch ($notafiscal->getIdstatus()) {
                case Notafiscal::STATUS_APROVADO:
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['NF_EMIT_APROV'], array('aprovada')));
                    break;
                case Notafiscal::STATUS_EMITIDO:
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['NF_EMIT_APROV'], array('emitida')));
                    break;
                case Notafiscal::STATUS_CANCELADO:
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['NF_EMIT_APROV'], array('cancelada')));
                }

                if (empty($corpoNF['CDESTAIBGEEMIT'])) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['COD_EST_EMI']);
                }
                $e_dscdfisopenf = $notafiscal->getDscdfisopenf();
                if (empty($e_dscdfisopenf)) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['NAT_OPE_DEV']);
                }
                $e_cdserie = $corpoNF['CDSERIE'];
                if (empty($e_cdserie)) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['SERIE_DEV']);
                }



                if ($paramgeral->getIdutdtproctnfe() == 'S') {
                    $filiOrig = Notafiscal::getRepository()->consultaFilialOrigem($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
                    $filiCent = Filial::getRepository()->consultaFilialCentralizadora($filiOrig['CDFILIORIG'], $notafiscal->getNrorg());

                    if (empty($filiCent['CDFILIFISC'])) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => $this->arrayMsgErros['SEM_FILI_CENT']);
                    } elseif (empty($filiCent['DTPROCESSA']) || $filiCent['DTPROCESSA'] == 0) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => $this->arrayMsgErros['DT_APROV_DEV']);
                    }
                }

                if ($paramgeral->getIddtemisdifdtbc() == 'S') {
                    if (!DateUtil::comparaDatas($corpoNF['DTEMISSAO'], DateUtil::getDataAtual(), DateUtil::IGUAL)) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['DTEMISSAO_DIFERE_BD'], array(DateUtil::getDataAtual(true))));
                    }
                }

                //Em caso de nota de devolução
                if ($corpoNF['IDOPERDEVONF'] == 'S' || $corpoNF['CDTIPOOPER'] == self::TIPO_OPERAC_COMPLEMENTO || $corpoNF['IDNFESTCANC'] == 'S') {
                    $notaDevolucao = Notafiscal::getRepository()->consultaNotaDevolucao($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());

                    if (is_array($notaDevolucao) && !empty($notaDevolucao)) {
                        foreach ($notaDevolucao as $erro) {
                            if (empty($erro['NRACESSONFE']) && empty($erro['CDSERIE'])) {
                                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                            'MENSAGEM'   => $this->arrayMsgErros['SERIE_ORIGEM']);
                            }
                        }
                    }
                }

                $verIncImposNFe = Itemproimpos::getRepository()->consultaItemProdutoImpostoNFE($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), self::ID_IMPOSTO_ISSQN, $notafiscal->getNrorg());
                if (is_array($verIncImposNFe) && !empty($verIncImposNFe)) {
                    if (empty($corpoNF['CDINSCMUNI'])) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => $this->arrayMsgErros['ISSQN_IM_DEV']);
                    } elseif (empty($corpoNF['CDCNAE'])) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => $this->arrayMsgErros['CNAE_DEV']);
                    }
                }

                if (empty($corpoNF['CDMODDOCFISC']) || $corpoNF['CDMODDOCFISC'] != self::ID_MODELO_DOC_NFE) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['CDMODDOCFISC_INCORRETO']);
                }

                if ($paramgeral->getIdutdtbcdtemiss() == 'N' && $paramgeral->getIdutdtproctnfe() == 'S') {
                    if (DateUtil::comparaDatas(DateUtil::getDataDeString($corpoNF['DTENTRSAID'], DateUtil::FORMATO_BRASILEIRO_DATAHORA), DateUtil::getDataDeString($corpoNF['DTEMISSAO'], DateUtil::FORMATO_BRASILEIRO_DATAHORA))) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => $this->arrayMsgErros['DATA_MAIOR']);
                    }
                }


                $this->validaEmitente($corpoNF, $notafiscal);
                $this->validaDestinatario($corpoNF, $notafiscal);
                $this->validaTransportadora($corpoNF, $notafiscal);
                $this->validaProdutorRural($corpoNF, $notafiscal);
                $this->validaItens($corpoNF, $notafiscal);
                $this->verificaDuplicata($corpoNF, $notafiscal);
            }
        }

        return $this->arrayErros;
    }

    private function getCorpoNf(\Fiscal\Entities\Notafiscal $notafiscal)
    {
        $cdfilial = $notafiscal->getIdentrsaidop() == 'S' ? $notafiscal->getCdfiliemit() : $notafiscal->getCdfilirece();

        /* @@var \Fiscal\Repositories\Notafiscal */
        return Notafiscal::getRepository()->getCorpoNotaFiscal(
            $cdfilial,
            $notafiscal->getNrlanctonf(),
            $notafiscal->getIdorigdestnf(),
            $notafiscal->getIdentrsaidop(),
            $notafiscal->getNrorg()
        );
    }

    private static function adicionaParametroMsgErro($mensagem, $param = array())
    {
        foreach ($param as $replace) {
            $mensagem = substr_replace($mensagem, $replace, strpos($mensagem, '%s'), 2);
        }
        return $mensagem;
    }

    /**
     * @param $corpoNF
     * @param $notafiscal
     */
    private function validaEmitente($corpoNF, Notafiscal $notafiscal)
    {
        if ($corpoNF['CDPAISBACENEMIT'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            $e_nrinsjuremit = trim(Util::removeMask($corpoNF['NRINSJUREMIT'], "/[^A-Za-z0-9 ]/"));
            if (!empty($e_nrinsjuremit)) {
                if (!ValidacaoNfEletronica::validaInscricao(trim(Util::removeMask($corpoNF['NRINSJUREMIT'], "/[^A-Za-z0-9 ]/")))) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_EMITENTE_INV']);
                }
            } else {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_EMITENTE_DEVE']);
            }
        }

        if (empty($corpoNF['CDPAISBACENEMIT'])) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['COD_PAIS_EMI']);
        }

        if (empty($corpoNF['DSENDEEMIT'])) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['END_EMI_DEV']);
        }

        if (empty($corpoNF['NMBAIREMIT'])) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['BAI_EMI']);
        }

        if ($corpoNF['CDPAISBACENEMIT'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            if (empty($corpoNF['CDMUNIIBGEEMIT'])) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['COD_MUN_EMI']);
            } elseif (strlen(trim(Util::removeMask($corpoNF['CDMUNIIBGEEMIT'], "/[^A-Za-z0-9 ]/"))) != 7) {
                if (strlen(trim($corpoNF['CDESTAIBGEEMIT']) . trim($corpoNF['CDMUNIIBGEEMIT'])) != 7) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['CDMUNEMI_INV']);
                }
            }

            if (empty($corpoNF['NMMUNIEMIT'])) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['MUN_EMI']);
            }
        }
        $e_nrcepemit = Util::removeMask($corpoNF['NRCEPEMIT'], "/[^A-Za-z0-9 ]/");
        if (empty($e_nrcepemit)) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['CEP_EMIT_DEV']);
        } elseif (!is_numeric(Util::removeMask($corpoNF['NRCEPEMIT'], "/[^A-Za-z0-9 ]/"))) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['CEP_EMIT_INC']);
        }
    }

    public static function validaInscricao($inscricao)
    {
        switch (strlen($inscricao)) {
        case(self::QTD_CHAR_CPF): {
            return (Util::validaCPF($inscricao) == true);
            break;
        }
        case(self::QTD_CHAR_CNPJ): {
            return (Util::validaCNPJ($inscricao) == true);
            break;
        }
        default:
            return false;
        }
    }

    /**
     * @param $notafiscal
     * @param $corpoNF
     */
    private function validaDestinatario($corpoNF, Notafiscal $notafiscal)
    {
        $cupomFiscal = Dadosnf::getRepository()->consultaCupomFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
        if (is_array($cupomFiscal) && !empty($cupomFiscal)) {
            if ($cupomFiscal['CDPAISBACEN'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
                if (trim(Util::removeMask($cupomFiscal['NRINSJUDESNF'], "/[^A-Za-z0-9 ]/"))) {
                    if (!ValidacaoNfEletronica::validaInscricao(trim(Util::removeMask($cupomFiscal['NRINSJUDESNF'], "/[^A-Za-z0-9 ]/")))) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_DESTINATARIO_INV']);
                    }
                } else {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_DESTINATARIO_DEVE']);
                }
            }
        } elseif ($corpoNF['CDPAISBACENCLIE'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            if (trim(Util::removeMask($corpoNF['NRINSJURCLIE'], "/[^A-Za-z0-9 ]/"))) {
                if (!ValidacaoNfEletronica::validaInscricao(trim(Util::removeMask($corpoNF['NRINSJURCLIE'], "/[^A-Za-z0-9 ]/")))) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_DESTINATARIO_INV']);
                }
            } else {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_DESTINATARIO_DEVE']);
            }
        }

        if (empty($corpoNF['CDPAISBACENCLIE'])) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['COD_PAIS_DES']);
        }
        if (empty($corpoNF['DSENDECLIE'])) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['END_DES_DEV']);
        }
        if (empty($corpoNF['NMBAIRCLIE'])) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['BAI_DES']);
        }
        if ($corpoNF['CDPAISBACENCLIE'] == self::ID_PAIS_BRASIL_BCO_CENTRAL) {
            if (empty($corpoNF['CDMUNIIBGECLIE'])) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['COD_MUN_DES']);
            } elseif (strlen(trim(Util::removeMask($corpoNF['CDMUNIIBGECLIE'], "/[^A-Za-z0-9 ]/"))) != 7 && strlen(trim(Util::removeMask($corpoNF['CDESTAIBGECLIE'] . $corpoNF['CDMUNIIBGECLIE'], "/[^A-Za-z0-9 ]/"))) != 7) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['COD_MUN_DES']);
            }
            if (empty($corpoNF['NMMUNICLIE'])) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['MUN_DES']);
            }
        }
        $e_nrcepclie = Util::removeMask($corpoNF['NRCEPCLIE'], "/[^A-Za-z0-9 ]/");
        if (empty($e_nrcepclie)) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['CEP_DEST_DEV']);
        } elseif (!is_numeric(Util::removeMask($corpoNF['NRCEPCLIE'], "/[^A-Za-z0-9 ]/"))) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['CEP_DEST_INC']);
        }
    }

    /**
     * @param $corpoNF
     * @param $notafiscal
     */
    private function validaTransportadora($corpoNF, Notafiscal $notafiscal)
    {
        $e_nrinsjurtran = trim(Util::removeMask($corpoNF['NRINSJURTRAN'], "/[^A-Za-z0-9 ]/"));
        if (!empty($e_nrinsjurtran)) {
            if (!ValidacaoNfEletronica::validaInscricao(trim(Util::removeMask($corpoNF['NRINSJURTRAN'], "/[^A-Za-z0-9 ]/")))) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_TRANSP_INV']);
            }
            if ($corpoNF['NRINSESTTRAN'] && !((strlen(Util::removeMask($corpoNF['NRINSESTTRAN'], "/[^A-Za-z0-9 ]/")) >= 2) && (strlen(Util::removeMask($corpoNF['NRINSESTTRAN'], "/[^A-Za-z0-9 ]/")) <= 14))) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['IE_TRA_INV']);
            }
            if ($corpoNF['IDPGTOFRETE'] == '') {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['MOD_FRETE_DEV']);
            } elseif ($corpoNF['IDPGTOFRETE'] && !in_array($corpoNF['IDPGTOFRETE'], array('0', '1', '2', '9'))) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['MOD_FRETE_INV']);
            }
            $e_nrplacaveic = Util::removeMask($corpoNF['NRPLACAVEIC'], "/[^A-Za-z0-9 ]/");
            if (!empty($e_nrplacaveic) || $corpoNF['SGUFVEIC']){
                $this->verificaPlacaVeiculo($corpoNF, $notafiscal);

                if (empty($corpoNF['SGUFVEIC'])) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['UF_VEI_TRA'], array($corpoNF['NMRAZSOCTRAN'])));
                }
            }
        }
    }

    /**
     * @param            $corpoNF
     * @param Notafiscal $notafiscal
     */
    private function verificaPlacaVeiculo($corpoNF, Notafiscal $notafiscal)
    {
        $e_nrplacaveic = Util::removeMask($corpoNF['NRPLACAVEIC'], "/[^A-Za-z0-9 ]/");
        if (empty($e_nrplacaveic)) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['PLA_VEI_TRA']);
        } else {
            $erroPlaca = false;
            $stNrPlaca = Util::removeMask($corpoNF['NRPLACAVEIC'], "/[^A-Za-z0-9 ]/");
            for ($i = 0; $i < strlen($stNrPlaca); $i++) {
                if ($i <= 2) {
                    if (!ctype_alpha($stNrPlaca[$i])) {
                        $erroPlaca = true;
                        break;
                    }
                } else {
                    if (!is_numeric($stNrPlaca[$i])) {
                        $erroPlaca = true;
                        break;
                    }
                }
            }
            if ($erroPlaca) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['PLA_VEI_TRA_INC'], array($corpoNF['NMRAZSOCTRAN'])));
            }
        }
    }

    /**
     * @param            $corpoNF
     * @param Notafiscal $notafiscal
     */
    private function validaProdutorRural($corpoNF, Notafiscal $notafiscal)
    {
        $nfProdRural = Nfprodrural::getRepository()->consultaNotaFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());

        if (is_array($nfProdRural) && !empty($nfProdRural)) {
            foreach ($nfProdRural as $prodRural) {
                $e_nrinsjuriest = trim(Util::removeMask($prodRural['NRINSJURIEST'], "/[^A-Za-z0-9 ]/"));
                if (!empty($e_nrinsjuriest)) {
                    if (!self::validaInscricao(trim(Util::removeMask($prodRural['NRINSJURIEST'], "/[^A-Za-z0-9 ]/")))) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_PRODRURAL_INV']);
                    }
                } else {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => $this->arrayMsgErros['CNPJ_CPF_PRODRURAL_DEVE']);
                }
                if (strlen(Util::removeMask($prodRural['NRINSJURIEST'], "/[^A-Za-z0-9 ]/") == 14)) {
                    $stNrInsProdRural = Util::mask(Util::removeMask(trim($prodRural['NRINSJURIEST']), "/[^A-Za-z0-9 ]/"), Util::picturePessoa('J'));
                } else {
                    if (strlen(Util::removeMask($prodRural['NRINSJURIEST'], "/[^A-Za-z0-9 ]/") == 11)) {
                        $stNrInsProdRural = Util::mask(Util::removeMask(trim($prodRural['NRINSJURIEST']), "/[^A-Za-z0-9 ]/"), Util::picturePessoa('F'));
                    } else {
                        $stNrInsProdRural = $prodRural['NRINSJURIEST'];
                    }
                }
                if (empty($prodRural['CDESTADOIBGE'])) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_IBGE_PROD_RURAL'], array($prodRural['NRNOTAFISC'], $stNrInsProdRural, $prodRural['CDSERIE'])));
                }
            }
        }
    }

    private function validaItens($corpoNF, Notafiscal $notafiscal)
    {
        $itemNota = Itemproduto::getRepository()->consultaItemNotaFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $corpoNF['CDPAISCLIE'], $corpoNF['SGESTACLIE'], $notafiscal->getNrorg());

        foreach ($itemNota as $item) {
            if (empty($item['NCM'])) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_NCM_DEV'], array($this->maskProduto($item['CDARVPROD']))));
            } elseif (!(strlen(Util::removeMask($item['NCM'], "/[^A-Za-z0-9 ]/")) == 2) ) {
                if (!(strlen(Util::removeMask($item['NCM'], "/[^A-Za-z0-9 ]/")) == 8) ) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_NCM_INC'], array($this->maskProduto($item['CDARVPROD']))));
                }
            }
            if (empty($item['CFOP'])) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CFOP_DEV'], array($this->maskProduto($item['CDARVPROD']))));
            } elseif (ValidaCFOPpadrao::validaCFOPpadrao($item['CFOP'], $item['NRORG']) != 'S') {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CFOP_INC'], array($this->maskProduto($item['CDARVPROD']))));
            }
            if ($item['CDSITUTRIBPR']) {
                if (strlen($item['CDSITUTRIBPR']) != 3) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CST_INV'], array($this->maskProduto($item['CDARVPROD']))));
                }
            } else {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CST_INV'], array($this->maskProduto($item['CDARVPROD']))));
            }
            if ($item['IDSEGMENTOPROD'] == 'MEDI' && empty($item['DTFABRPROD'])) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['DAT_FAB_LOT'], array($this->maskProduto($item['CDARVPROD']))));
            }
            if ($item['CDBARPRODUTO']) {
                if (!(strlen($item['CDBARPRODUTO']) == 8) && !(strlen($item['CDBARPRODUTO']) == 12) && !(strlen($item['CDBARPRODUTO']) == 13) && !(strlen($item['CDBARPRODUTO']) == 14)) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CEAN_INV'], array($this->maskProduto($item['CDARVPROD']))));
                }
            }

            $paramEmpresa = Paramempresa::getRepository()->consultaParamEmpresa($corpoNF['CDEMPRESA'], $corpoNF['NRORG']);
            if ($paramEmpresa['CDSITUCRT'] == '1') {
                if (empty($item['CDFISCOPERSN'])) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CSOSN_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                }
                if ($item['CSOSN'] && strlen($item['CSOSN']) != 4) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_CSOSN_INV'], array($this->maskProduto($item['CDARVPROD']))));
                }
            }

            $verifAdicoes = Adicdecimpnfe::getRepository()->verificaAdicoes($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $item['NRSEQITPROD'], $notafiscal->getNrorg());
            if ($verifAdicoes > 100) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => $this->arrayMsgErros['ADICOES_MAIOR_120']);
            }

            $this->validaImpostosItens($corpoNF, $notafiscal, $item, $paramEmpresa);
        }
    }

    private function maskProduto($cdarvprod)
    {
        return Util::mask($cdarvprod, preg_replace("/([^.])/", '#', PictureProduto::mascaraProduto()));
    }

    private function validaImpostosItens($corpoNF, Notafiscal $notafiscal, $item, $paramEmpresa)
    {
        $impostosItem = Itemproimpos::getRepository()->consultaImpostosItem($item['CDFILIAL'], $item['NRLANCTONF'], $item['NRSEQITPROD'], $item['NRORG']);
        $impostosNFE  = Impostonfe::getRepository()->retornaImpostosNFE($item['NRORG']);

        $impostosNfeICMS = end(array_filter($impostosNFE, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "00";')));

        if (empty($impostosNfeICMS)) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['ICMS_N_RELAC'], array($this->maskProduto($item['CDARVPROD']))));
        } else {
            $impostosItemICMS = end(array_filter($impostosItem, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "00";')));

            if (!empty($impostosItemICMS)) {
                //Só valida o CST do ICMS caso o Código do Regime Tributário da empresa seja "3 - Regime Normal"
                if ($paramEmpresa['CDSITUCRT'] == self::REGIME_NORMAL) {
                    if (empty($impostosItemICMS['CST'])) {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CST_ICMS_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                    }
                    if ($impostosItemICMS['CST'] == '00' || $impostosItemICMS['CST'] == '10' || $impostosItemICMS['CST'] == '20' || $impostosItemICMS['CST'] == '30'
                        || $impostosItemICMS['CST'] == '51' || $impostosItemICMS['CST'] == '70' || $impostosItemICMS['CST'] == '90'
                    ) {
                        if ($impostosItemICMS['CST'] != '30' && empty($impostosItemICMS['IDMODBCITNFE'])) {
                            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                        'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['MOD_BC_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                        }
                        if ($impostosItemICMS['IDMODBCSTITNFE'] == '10' || $impostosItemICMS['IDMODBCSTITNFE'] == '30' || $impostosItemICMS['IDMODBCSTITNFE'] == '70' || $impostosItemICMS['IDMODBCSTITNFE'] == '90') {
                            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                        'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['MOD_BCST_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                        }
                    } elseif ($impostosItemICMS['CST'] != '40' || $impostosItemICMS['CST'] != '41' || $impostosItemICMS['CST'] != '50') {
                        if ($impostosItemICMS['IDIMPDESONERAIT'] == 'S' && $impostosItemICMS['IDMOTDESONERAIT'] == '7') {
                            if (empty($corpoNF['NRINSCSUFRAMA'])) {
                                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                            'MENSAGEM'   => $this->arrayMsgErros['INSC_SUFRAMA_DEV']);
                            }
                            if (!$item['CFOP'] == '6109' || !$item['CFOP'] == '6110') {
                                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CFOP_INVA_SUFRAMA'], array($this->maskProduto($item['CDARVPROD']))));
                            }
                        }
                    }
                } else {
                    //Se ele for simples e o CSOSN usar BC e tiver ICMS valida a BC do ICMS.
                    if ($item['CDFISCOPERSN'] == '201' || $item['CDFISCOPERSN'] == '202' || $item['CDFISCOPERSN'] == '203' || $item['CDFISCOPERSN'] == '900') {
                        if (empty($impostosItemICMS['IDMODBCITNFE'])) {
                            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                        'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['MOD_BCST_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                        }
                    }
                }
            } else {
                //Imposto ISSQN
                $impostosNfeISSQN = end(array_filter($impostosNFE, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "31";')));

                // Foi verificado que existe ISSQN no simples também, logo as validações abaixo serão validas.
                if (empty($impostosNfeISSQN)) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['ISSQN_N_RELAC'], array($this->maskProduto($item['CDARVPROD']))));
                } else {
                    $impostosItemISSQN = end(array_filter($impostosItem, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "31";')));
                    if (!empty($impostosItemISSQN)) {
                        //Verifica o codigo para classificação do serviço
                        $classificacaoProduto = Produto::getRepository()->consultaClassificacaoServico($item['CDARVPROD']);

                        if (empty($classificacaoProduto['CDCLASSPROD'])) {
                            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                        'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CLA_SER_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                        }
                        if (empty($classificacaoProduto['CDTRIBISSQN'])) {
                            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                        'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COD_TRIB_ISSQN_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                        }
                    } else {
                        $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                    'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                    'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['ICMS_OU_ISS_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                    }
                }
            }
        }

        //Imposto IPI
        $impostosItemIPI = end(array_filter($impostosItem, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "10";')));

        if (!empty($impostosItemIPI)) {
            //Só valida o CST do IPI caso esteja vazio e o Código do Regime Tributário da empresa seja "3 - Regime Normal"
            if ($paramEmpresa['CDSITUCRT'] == '3') {
                if (empty($impostosItemIPI['CST'])) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CST_IPI_DEV'], array($this->maskProduto($item['CDARVPROD']))));
                } elseif (!in_array($impostosItemIPI['CST'], array('00', '01', '02', '03', '04', '05', '49', '50', '51', '52', '53', '54', '55', '99'))) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CST_IPI_INCORRETO'], array($this->maskProduto($item['CDARVPROD']))));
                }
            }
        } else {
            if ($item['CFOP'] && $item['CFOP'] == '3') {
                if ($item['CFOP'] != '3201' || $item['CFOP'] != '3202' || $item['CFOP'] != '3211' || $item['CFOP'] != '3503' || $item['CFOP'] != '3553') {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CFOP_IMP_INC_IPI'], array($this->maskProduto($item['CDARVPROD']))));
                }
            }
        }

        // Verificando se existe incidência de ICMS, IPI e ISSQN para o mesmo produto.
        if (!empty($impostosItemISSQN)) {
            if (!empty($impostosItemICMS) || !empty($impostosItemIPI)) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CONFLITO_INCIDENCIA'], array($this->maskProduto($item['CDARVPROD']))));
            }
        }
        if ($item['CFOP'] && $item['CFOP'] == '3') {
            if ($item['CFOP'] != '3201' || $item['CFOP'] != '3202' || $item['CFOP'] != '3211' || $item['CFOP'] != '3503' || $item['CFOP'] != '3553') {
                //Imposto II
                $impostosItemII = end(array_filter($impostosItem, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "20";')));

                if (empty($impostosItemII)) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CFOP_IMP_INC_II'], array($this->maskProduto($item['CDARVPROD']))));
                }
            }
        }

        //Imposto PIS
        $impostosNfePIS = end(array_filter($impostosNFE, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "50";')));

        if (empty($impostosNfePIS)) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['PIS_N_RELAC'], array($this->maskProduto($item['CDARVPROD']))));
        } else {
            $impostosItemPIS = end(array_filter($impostosItem, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "50";')));
            if (empty($impostosItemPIS)) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['PIS_DEV'], array($this->maskProduto($item['CDARVPROD']))));
            } elseif ($impostosItemPIS['CST']) {
                if (!in_array(
                    $impostosItemPIS['CST'], array('01', '02', '03', '04', '05', '06', '07', '08', '09', '49', '50', '51', '52', '53', '54', '55', '56',
                        '60', '61', '62', '63', '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99')
                )
                ) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CST_PIS_INCORRETO'], array($this->maskProduto($item['CDARVPROD']))));
                }
            } else {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CST_PIS_DEV'], array($this->maskProduto($item['CDARVPROD']))));
            }
        }

        //Imposto COFINS
        $impostosNfeCOFINS = end(array_filter($impostosNFE, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "60";')));

        if (empty($impostosNfeCOFINS)) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COFINS_N_RELAC'], array($this->maskProduto($item['CDARVPROD']))));
        } else {
            $impostosItemCOFINS = end(array_filter($impostosItem, create_function('$imposto', 'return $imposto["CDIMPOSTONFE"] == "60";')));

            if (empty($impostosItemCOFINS)) {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['COF_DEV'], array($this->maskProduto($item['CDARVPROD']))));
            } elseif ($impostosItemCOFINS['CST']) {
                if (!in_array(
                    $impostosItemCOFINS['CST'], array('01', '02', '03', '04', '05', '06', '07', '08', '09', '49', '50', '51', '52', '53', '54', '55', '56',
                        '60', '61', '62', '63', '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99')
                )
                ) {
                    $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                                'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                                'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CST_COF_INCORRETO'], array($this->maskProduto($item['CDARVPROD']))));
                }
            } else {
                $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                            'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                            'MENSAGEM'   => self::adicionaParametroMsgErro($this->arrayMsgErros['CST_COF_DEV'], array($this->maskProduto($item['CDARVPROD']))));
            }
        }
    }

    private function verificaDuplicata($corpoNF, Notafiscal $notafiscal)
    {
        $duplicata = Contareceber::getRepository()->consultaQuantidadeDuplicata($corpoNF['CDFILIAL'], $corpoNF['NRLANCTONF'], $corpoNF['NRORG']);

        if ($duplicata[0]['QTDDUPLICATA'] > 120) {
            $this->arrayErros[] = array('CDFILIAL'   => $notafiscal->getCdfilial(),
                                        'NRLANCTONF' => $notafiscal->getNrlanctonf(),
                                        'MENSAGEM'   => $this->arrayMsgErros['DUPLI_MAIOR_120']);
        }
    }
}

